# -*- coding: utf-8 -*-

from django.apps import apps
from django.contrib import admin
from django.utils.translation import ugettext as _

from image_cropping import ImageCroppingMixin

from . import models


app_models = apps.get_app_config('product').get_models()


class ProductCategoryAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general information'), {
            'fields': (
                'code', 'name', 'slug',
                'parent', 'related_categories',
                'description'
            )
        }),
        (_('visualization admin'), {
            'fields': ('ordering', 'status'),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = (
        'get_status_display', 'code', 'name', 'ordering',
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)
    prepopulated_fields = {"slug": ("name",)}


class UnitTypeAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general information'), {
            'fields': ('code', 'name',)
        }),
        (_('visualization admin'), {
            'fields': ('ordering', 'status'),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = (
        'get_status_display', 'code', 'name', 'ordering',
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)


class ProductAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general admin'), {
            'fields': (
                'code', 'name', 'slug', 'categories',
                'production_code', 'production_year',
                'vat', 'store_cause', 'vendor', 'unit', 'min_supply',
                'short_description', 'full_description',
                'note'
            )
        }),
        (_('visualization information'), {
            'fields': ('ordering', 'status'),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = (
        'get_status_display', 'code', 'name', 'ordering',
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)
    prepopulated_fields = {"slug": ("name",)}


class MediaAdmin(ImageCroppingMixin, admin.ModelAdmin):
    fieldsets = (
        (_('general information'), {
            'fields': (
                'title', 'slug', 'product', 'media',
                'small_cropping', 'big_cropping',
                'url', 'description',
            )
        }),
        (_('visualization admin'), {
            'fields': ('ordering',),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = (
        'title', 'product',
        'get_media_link', 'get_small_cropping_link', 'get_big_cropping_link',
        'ordering',
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)
    prepopulated_fields = {"slug": ("title",)}

    def get_media_link(self, obj):
        if obj.get_url():
            return """
                <a href='{0}' title='{1}' target='_blank'>
                    <img src='{2}' title='{1}' alt='{1}'
               </a>""".format(
                    obj.get_url(), obj.title, obj.get_icon_url()
                )
        return ""
    get_media_link.short_description = _('media')
    get_media_link.allow_tags = True

    def get_small_cropping_link(self, obj):
        if obj.get_small_url():
            return """
                <a href='{0}' title='{1}' target='_blank'>
                    <img src='{2}' title='{1}' alt='{1}'
               </a>""".format(
                    obj.get_small_url(), obj.title, obj.get_icon_url()
                )
        return ""
    get_small_cropping_link.short_description = _('small')
    get_small_cropping_link.allow_tags = True

    def get_big_cropping_link(self, obj):
        if obj.get_big_url():
            return """
                <a href='{0}' title='{1}' target='_blank'>
                    <img src='{2}' title='{1}' alt='{1}'
               </a>""".format(
                    obj.get_big_url(), obj.title, obj.get_icon_url()
                )
        return ""
    get_big_cropping_link.short_description = _('big')
    get_big_cropping_link.allow_tags = True


class ProductColorAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general information'), {
            'fields': (
                'code_rgb', 'name', 'product',
            )
        }),
        (_('visualization admin'), {
            'fields': ('status', 'ordering',),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = (
        'get_status_display', 'code_rgb', 'name', 'product', 'ordering',
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)


class ProductSizeAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general information'), {
            'fields': ('code', 'name', 'slug',)
        }),
        (_('visualization admin'), {
            'fields': ('ordering', 'status'),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = (
        'get_status_display', 'code', 'name', 'ordering',
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)
    prepopulated_fields = {"slug": ("name",)}


class ProductProductSizeJoinAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general information'), {
            'fields': ('product', 'product_size',)
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = (
        'product', 'product_size',
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)


class ProductSeasonAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general information'), {
            'fields': ('product', 'is_spring', 'is_summer', 'is_autumn', 'is_winter')
        }),
        (_('visualization admin'), {
            'fields': ('ordering', 'status'),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = (
        'get_status_display', 'product',
        'is_spring', 'is_summer', 'is_autumn', 'is_winter', 'ordering',
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)


admin.site.register(models.ProductCategory, ProductCategoryAdmin)
admin.site.register(models.UnitType, UnitTypeAdmin)
admin.site.register(models.Product, ProductAdmin)
admin.site.register(models.Media, MediaAdmin)
admin.site.register(models.ProductColor, ProductColorAdmin)
admin.site.register(models.ProductSize, ProductSizeAdmin)
admin.site.register(models.ProductProductSizeJoin, ProductProductSizeJoinAdmin)
admin.site.register(models.ProductSeason, ProductSeasonAdmin)
