# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six import python_2_unicode_compatible
import os

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from easy_thumbnails.files import get_thumbnailer
from image_cropping import ImageRatioField

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel

from thux.vat.models import Vat
from thux.store.models import StoreCause


def get_upload_media_path(instance, filename):
    media_dir = instance.product.slug[0:3]
    ext = filename.split('.')[-1]
    filename = '{}.{}'.format(instance.product.slug, ext)
    return os.path.join('product', 'media', '{0}'.format(media_dir), filename)


@python_2_unicode_compatible
class ProductCategory(DateModel, UserModel, StatusModel, OrderedModel):
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_('code'),
    )
    name = models.CharField(
        unique=True,
        max_length=255,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        unique=True,
        max_length=255,
        verbose_name=_('slug'),
    )
    parent = models.ForeignKey(
        'self',
        blank=True, null=True,
        related_name='children',
    )
    description = models.TextField(
        blank=True,
        verbose_name=_('description'),
    )
    related_categories = models.ManyToManyField(
        'self',
        symmetrical=False,
        blank=True,
        verbose_name=_('related categories'),
    )

    class Meta:
        verbose_name = _('product category')
        verbose_name_plural = _('product categories')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_productcategory", "Can list product category"),
            ("detail_productcategory", "Can detail product category"),
            ("disable_productcategory", "Can disable product category"),
        )

    def __str__(self):
        return '{} {}'.format(self.code, self.name)


@python_2_unicode_compatible
class UnitType(DateModel, UserModel, StatusModel, OrderedModel):
    code = models.CharField(
        primary_key=True,
        max_length=3,
        verbose_name=_('code'),
    )
    name = models.CharField(
        db_index=True,
        max_length=255,
        verbose_name=_('name'),
    )

    class Meta:
        verbose_name = _('unit')
        verbose_name_plural = _('units')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_unittype", "Can list unit type"),
            ("detail_unittype", "Can detail unit type"),
            ("disable_unittype", "Can disable unit type"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Product(DateModel, UserModel, StatusModel, OrderedModel):
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_('code'),
    )
    name = models.CharField(
        db_index=True,
        max_length=255,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        unique=True,
        max_length=255,
        verbose_name=_('slug'),
    )
    categories = models.ManyToManyField(
        ProductCategory,
        blank=True,
        related_name='products',
        verbose_name=_('categories'),
    )
    vat = models.ForeignKey(
        Vat,
        verbose_name=_('vat'),
        related_name='products',
    )
    store_cause = models.ForeignKey(
        StoreCause,
        blank=True, null=True,
        related_name='products',
        verbose_name=_('store cause')
    )
    production_code = models.CharField(
        max_length=25,
        blank=True,
        db_index=True,
        verbose_name=_('production code'),
    )
    production_year = models.PositiveSmallIntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('production year')
    )
    vendor = models.CharField(
        max_length=200,
        blank=True,
        db_index=True,
        verbose_name=_('vendor'),
    )
    unit = models.ForeignKey(
        UnitType,
        db_index=True,
        verbose_name=_('unit'),
    )
    short_description = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_('short description'),
    )
    full_description = models.TextField(
        blank=True,
        verbose_name=_('full description'),
    )
    min_supply = models.IntegerField(
        default=0,
        verbose_name=_('min supply'),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_('note'),
    )

    class Meta:
        verbose_name = _('product')
        verbose_name_plural = _('products')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_product", "Can list product"),
            ("detail_product", "Can detail product"),
            ("disable_product", "Can disable product"),
        )

    def __str__(self):
        return '{} {}'.format(self.code, self.name)

    def get_first_media(self):
        medias = self.medias.all()
        if medias:
            return medias[0]
        return None

    def get_price_vat_included(self):
        # TODO: da sistemare prezzi correnti in funzione dell'utente
        if self.prices.all():
            return self.prices.all()[0].get_price_vat_included()
        return None


@python_2_unicode_compatible
class Media(DateModel, UserModel, OrderedModel):
    title = models.CharField(
        unique=True,
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        unique=True,
        verbose_name=_('slug'),
        max_length=255,
    )
    product = models.ForeignKey(
        Product,
        related_name='medias',
        verbose_name=_('product'),
    )
    media = models.ImageField(
        blank=True, null=True,
        verbose_name=_("media"),
        upload_to=get_upload_media_path,
    )
    small_cropping = ImageRatioField(
        'media', '300x300'
    )
    big_cropping = ImageRatioField(
        'media', '600x600'
    )
    url = models.URLField(
        blank=True, null=True,
        verbose_name=_('url')
    )
    description = models.TextField(
        verbose_name=_('description'),
    )

    class Meta:
        verbose_name = _('media')
        verbose_name_plural = _('media')
        ordering = ('ordering', 'product__name', 'id',)
        permissions = (
            ("list_media", "Can list media"),
            ("detail_media", "Can detail media"),
        )

    def __str__(self):
        return self.product.name

    def get_thumbnail(
            self, width=settings.PRODUCT_MEDIA_THUMBNAIL_WIDTH,
            height=settings.PRODUCT_MEDIA_THUMBNAIL_HEIGHT):
        if self.media:
            return get_thumbnailer(
                self.media
            ).get_thumbnail({
                'size': (width, height),
                'crop': True,
                'detail': True,
            }).url

    def get_url(self):
        if self.media:
            return self.media.url
        elif self.url:
            return self.url
        else:
            return None

    def get_small_url(self):
        if self.small_cropping:
            return self.get_thumbnail(300, 300)
        return None

    def get_big_url(self):
        if self.small_cropping:
            return self.get_thumbnail(600, 600)
        return None

    def get_icon_url(self):
        if self.media:
            return self.get_thumbnail(width=50, height=50)
        return None


@python_2_unicode_compatible
class ProductColor(DateModel, UserModel, StatusModel, OrderedModel):
    code_rgb = models.CharField(
        max_length=6,
        verbose_name=_('code RGB'),
    )
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    product = models.ForeignKey(
        Product,
        related_name='colors',
        verbose_name=_('product'),
    )

    class Meta:
        verbose_name = _('color')
        verbose_name_plural = _('colors')
        ordering = ('ordering', 'product__name', 'id',)
        unique_together = (
            ('name', 'product',),
            ('code_rgb', 'product',),
        )
        permissions = (
            ("list_productcolor", "Can list product color"),
            ("detail_productcolor", "Can detail product color"),
        )

    def __str__(self):
        return "{}: {}".format(self.product.name, self.name)


@python_2_unicode_compatible
class ProductSize(DateModel, UserModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=6,
        unique=True,
        verbose_name=_('code'),
    )
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        unique=True,
        max_length=255,
        verbose_name=_('slug'),
    )

    class Meta:
        verbose_name = _('size')
        verbose_name_plural = _('sizes')
        ordering = ('ordering', 'name',)
        permissions = (
            ("list_productsize", "Can list product size"),
            ("detail_productsize", "Can detail product size"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class ProductProductSizeJoin(DateModel, UserModel):
    product = models.OneToOneField(
        Product,
        related_name='product_size_join'
    )
    product_size = models.ForeignKey(
        ProductSize,
        related_name='product_joins'
    )

    class Meta:
        verbose_name = _('product - product size join')
        verbose_name_plural = _('product - product size join')
        permissions = (
            ("list_productproductsizejoin", "Can list product - product size join"),
            ("detail_productproductsizejoin", "Can detail product - product size join"),
        )

    def __str__(self):
        return "{} {}".format(self.product, self.product_size)


@python_2_unicode_compatible
class ProductSeason(DateModel, UserModel, StatusModel, OrderedModel):
    product = models.OneToOneField(
        Product,
        related_name='season',
    )
    is_spring = models.BooleanField(
        default=False,
        verbose_name=_('spring'),
    )
    is_summer = models.BooleanField(
        default=False,
        verbose_name=_('summer'),
    )
    is_autumn = models.BooleanField(
        default=False,
        verbose_name=_('autumn'),
    )
    is_winter = models.BooleanField(
        default=False,
        verbose_name=_('winter'),
    )

    class Meta:
        verbose_name = _('season')
        verbose_name_plural = _('seasons')
        ordering = ('ordering', 'product__name',)
        permissions = (
            ("list_productseason", "Can list product season"),
            ("detail_productseason", "Can detail product season"),
        )

    def __str__(self):
        return self.product.name
